import{s as r,b as e,a,S as s}from"./chunk-AEK57VVT-kQQtlouG.js";import{_ as i}from"./mermaid-vendor-CY273lNM.js";import"./chunk-RZ5BOZE2-BJpHTKlK.js";import"./feature-graph-C2oTJZJ8.js";import"./react-vendor-DEwriMA6.js";import"./graph-vendor-B-X5JegA.js";import"./ui-vendor-CeCm8EER.js";import"./utils-vendor-BysuhMZA.js";var b={parser:a,get db(){return new s(2)},renderer:e,styles:r,init:i(t=>{t.state||(t.state={}),t.state.arrowMarkerAbsolute=t.arrowMarkerAbsolute},"init")};export{b as diagram};
